# -*- perl -*-

#
#   Copyright (C) Heinz-Josef Claes (2001)
#                 hjclaes@web.de
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#   
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#


push @VERSION, '$Id: readKeyFromFile.pl,v 1.2 2002/07/03 05:54:51 hjc Exp $ ';

use strict;

require 'checkObjPar.pl';

package readKeyFromFile;

##################################################
sub new
{
    my ($class) = shift;
    my ($self) = {};

    # Defaultwerte f"ur Parameter setzen
    my (%params) = ('-filename'       => undef,
		    '-delimiter'      => ':',    # Trenner Key -> value
		    '-prLog'          => undef);

    &::checkObjectParams(\%params, \@_, 'ReadKeyFromFile::new',
			 ['-filename', '-prLog']);

    &::setParamsDirect($self, \%params);

    my $file = $params{'-filename'};
    my $prLog = $params{'-prLog'};
    local *FILE;
    open(FILE, "< $file") or
	$prLog->print('-kind' => 'E',
		      '-str' => ["cannot open <$file>, exiting"],
		      '-exit' => 1);
    $self->{'filehandle'} = *FILE;

    bless $self, $class;
}


##################################################
sub readKey
{
    my $self = shift;

    my $delimiter = $self->{'-delimiter'};
    if ($self->{'readReady'})
    {
	$self->{'readReady'} = undef;
	return ();
    }

    $self->{'lastReadKey'} = ''
	unless (defined $self->{'lastReadKey'});

    local *FILE = $self->{'filehandle'};

    my ($l, $keyline, $lineno);
    my $keyFound = undef;
    while (1)
    {
	if ($self->{'lastReadKey'} ne '')
	{
	    $l = $self->{'lastReadKey'};
	    $self->{'lastReadKey'} = '';
	    $lineno = $self->{'lastReadKeyLineno'};
	}
	else
	{
	    $l = <FILE>;
	    unless (defined $l)
	    {
		$self->{'lastReadKey'} = undef;
		$self->{'readReady'} = 1;
		$lineno = $self->{'lastReadKeyLineno'};
		return ($lineno, $keyline);
	    }
	    chomp $l;
	    $l =~ s/\#.*//;            # Kommentar weg
	    next if $l =~ /^\s*$/o;     # Leerzeilen berlesen
	}
	if ($l =~ /^(\S+)$delimiter/)   # neuer key
	{
	    if ($keyFound)             # key schon gefunden, das war's
	    {
		$self->{'lastReadKey'} = $l;
		$self->{'lastReadKeyLineno'} = $.;
		return ($lineno, $keyline);
	    }
	    else
	    {
		$keyline .= ' ' . $l;
		$keyFound = 1;
		$lineno = $.;
	    }
	}
	else
	{
	    $keyline .= ' ' . $l;
	}
    }
}

##################################################
sub DESTROY
{
    my ($self) = shift;

    local *FILE = $self->{'filehandle'};
    close(FILE) or
	$self->{'prLog'}->print('-kind' => 'E',
				'-str' =>
				["cannot close <" .
				 $self->{'param'}{'-file'} . ">\n"]);
}

1
